#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ogcsys.h>
#include <gccore.h>
#include <wiiuse/wpad.h>
#include <zlib.h>
#include <fat.h>
#include <mxml.h>

static u32 *xfb;
static GXRModeObj *rmode;

bool setting_background_music = false;
bool setting_rumble = true;
bool setting_tips = true;

void Initialise() {
	
	VIDEO_Init();
	WPAD_Init();
	
	rmode = VIDEO_GetPreferredMode(NULL);
	
	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	console_init(xfb,20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);
	
	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();
	
}

void update_settings() {
	mxml_node_t *xml;
	mxml_node_t *data; 
	xml = mxmlNewXML("1.0");
	
	data = mxmlNewElement(xml, "settings");
	
	char set1[1];
	sprintf(set1, "%i", setting_background_music);
	mxmlElementSetAttr(data, "setting_background_music", set1);
	char set2[1];
	sprintf(set2, "%i", setting_rumble);
	mxmlElementSetAttr(data, "setting_rumble", set2);
	char set3[1];
	sprintf(set3, "%i", setting_tips);
	mxmlElementSetAttr(data, "setting_tips", set3);
	
	FILE *f;
	f = fopen("sd:/settings.xml", "wb");
	
	if (f == NULL) {
		fclose(f);
		printf("Settings could not be written.\n");
	}
	else {
		mxmlSaveFile(xml, f, MXML_NO_CALLBACK);
		fclose(f);
		mxmlDelete(data);
		mxmlDelete(xml);
		printf("Settings Saved\n\n");
	}
}

void load_settings() {
	mxml_node_t *tree;
	mxml_node_t *data;
	
	FILE *fp = fopen("sd:/settings.xml", "rb");
	if (fp == NULL) {
		fclose(fp);
	}
	else {
		fseek (fp , 0, SEEK_END);
		long settings_size = ftell (fp);
		rewind (fp);
		
		if (settings_size > 0) {
			
			tree = mxmlLoadFile(NULL, fp, MXML_NO_CALLBACK);
			fclose(fp);
			
			data = mxmlFindElement(tree, tree, "settings", NULL, NULL, MXML_DESCEND);
			
			if (mxmlElementGetAttr(data,"setting_background_music")) {
				setting_background_music = atoi(mxmlElementGetAttr(data,"setting_background_music"));
				printf("Setting for background music loaded\n");
			}
			if (mxmlElementGetAttr(data,"setting_rumble")) {
				setting_rumble = atoi(mxmlElementGetAttr(data,"setting_rumble"));
				printf("Setting for rumble loaded\n");
			}
			if (mxmlElementGetAttr(data,"setting_tips")) {
				setting_tips = atoi(mxmlElementGetAttr(data,"setting_tips"));
				printf("Setting for tips loaded\n");
			}
			
			mxmlDelete(data);
			mxmlDelete(tree);
			printf("Settings loaded.\n\n");
		}
		else {
			fclose(fp);
			unlink("sd:/settings.xml");
		}
	}
}

void print_settings() {
	if (setting_background_music == true) {
		printf("Background music setting is enabled\n");
	}
	else {
		printf("Background music setting is disabled\n");
	}
	
	if (setting_rumble == true) {
		printf("Rumble setting is enabled\n");
	}
	else {
		printf("Rumble setting is disabled\n");
	}
	
	if (setting_tips == true) {
		printf("Tips setting is enabled\n");
	}
	else {
		printf("Tips setting is disabled\n");
	}
}


int main() {
	
	Initialise();
	printf("\x1b[2;0H");
	
	if (!fatInitDefault()) {
		printf("Unable to initialise FAT subsystem, exiting\n");
		exit(0);
	}
	
	printf("Tutorial 13: XML Settings\n");
	printf("Press 1 to load settings. Press 2 to save settings. Press A to display settings.\n\n");
	
	while(1) {
		
		WPAD_ScanPads();
		u32 pressed = WPAD_ButtonsDown(0);
		
		if (pressed & WPAD_BUTTON_1) {
			load_settings();
		}
		
		if (pressed & WPAD_BUTTON_2) {
			update_settings();
		}
		
		if (pressed & WPAD_BUTTON_A) {
			print_settings();
		}
		
		if (pressed & WPAD_BUTTON_HOME) {
			exit(0);
		}
		
		VIDEO_WaitVSync();
	}
	
	return 0;
}
